/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.antragsraum.events.NachrichtEventTestFactory;
import lombok.extern.log4j.Log4j2;

@SpringBootTest
@SpringJUnitConfig(classes = { CommandRemoteServiceTestConfiguration.class })
@DirtiesContext
@ExtendWith(MockitoExtension.class)
@Log4j2
@Disabled("needs to be fixed when switched to rest")
class CommandRemoteServiceITCase {
	private static final String ADDRESS = "in-process:test2";

	@SpyBean
	CommandRemoteService commandRemoteService;

	@Nested
	class TestGetCommand {
		private final NachrichtEvent nachrichtEvent = NachrichtEventTestFactory.createNachrichtEventBuilder().address(ADDRESS).build();

		@BeforeEach
		void setup() {
			UsernamePasswordAuthenticationToken authentication = mock(UsernamePasswordAuthenticationToken.class);
			lenient().when(authentication.getCredentials()).thenReturn("jwt-token");
			SecurityContextHolder.getContext().setAuthentication(authentication);
		}

		@Test
		void shouldLoadCommand() {
			var command = commandRemoteService.getCommand(nachrichtEvent, CommandTestFactory.ID);

			assertThat(command).isInstanceOf(Command.class);
		}

		@Test
		void shouldHaveCommandId() {
			var command = commandRemoteService.getCommand(nachrichtEvent, CommandTestFactory.ID);

			assertThat(command.id()).isEqualTo(CommandTestFactory.ID);
		}

		@Test
		void shouldHaveNachrichtEventId() {
			var command = commandRemoteService.getCommand(nachrichtEvent, CommandTestFactory.ID);

			assertThat(command.nachrichtEventId()).isEqualTo(NachrichtEventTestFactory.ID);
		}

		@Test
		void shouldHaveStatus() {
			var command = commandRemoteService.getCommand(nachrichtEvent, CommandTestFactory.ID);

			assertThat(command.status()).isEqualTo(CommandTestFactory.STATUS);
		}

		@Test
		void shouldHaveCreatedAt() {
			var command = commandRemoteService.getCommand(nachrichtEvent, CommandTestFactory.ID);

			assertThat(command.createdAt()).isEqualTo(CommandTestFactory.CREATED_AT_DATE);
		}

		@Test
		void shouldHaveFinishedAt() {
			var command = commandRemoteService.getCommand(nachrichtEvent, CommandTestFactory.ID);

			assertThat(command.finishedAt()).isEqualTo(CommandTestFactory.FINISHED_AT_DATE);
		}
	}
}
