/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.antragsraum.events.NachrichtEventService;
import de.ozgcloud.antragsraum.events.NachrichtEventTestFactory;

@ExtendWith(MockitoExtension.class)
class CommandServiceTest {
	@Spy
	@InjectMocks
	private CommandService commandService;
	@Mock
	private CommandRemoteService commandRemoteService;
	@Mock
	private NachrichtEventService nachrichtEventService;

	@Nested
	class TestGetCommand {
		@BeforeEach
		void init() {
			final NachrichtEvent nachrichtEvent = NachrichtEventTestFactory.createNachrichtEvent();
			when(nachrichtEventService.getNachrichtEventById(NachrichtEventTestFactory.ID)).thenReturn(nachrichtEvent);
			when(commandRemoteService.getCommand(nachrichtEvent, CommandTestFactory.ID)).thenReturn(CommandTestFactory.createCommand());
		}

		@Test
		void shouldReturnCommand() {
			var result = commandService.getCommand(CommandTestFactory.ID, NachrichtEventTestFactory.ID);

			assertThat(result).isInstanceOf(Command.class);
		}
	}
}