/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestClient;

import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcFindRueckfragenResponse;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcGetRueckfrageResponse;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcSendRueckfrageAnswerResponse;

@ExtendWith(MockitoExtension.class)
@Disabled("Needs to be updated to the POST requests")
class NachrichtenRestClientTest {
	private final static String ADDRESS = "http://localhost:8382";
	private final static String TOKEN = "saml_token";

	@Spy
	@InjectMocks
	private NachrichtenRestClient restClient;

	@Mock
	private RestClient nachrichtRestClient;
	@Mock
	private AntragraumProxyMapper mapper;

	@Nested
	class TestSendingRequests {

		@Nested
		class TestRueckfrageAnswer {
			@SuppressWarnings("unchecked")
			@BeforeEach
			void setup() {
				var bodyUriSpec = mock(RestClient.RequestBodyUriSpec.class);
				var bodyContentTypeSpec = mock(RestClient.RequestBodySpec.class);
				var bodySpec = mock(RestClient.RequestBodySpec.class);
				var retrieveSpec = mock(RestClient.RequestBodySpec.class);
				var responseSpec = mock(RestClient.ResponseSpec.class);
				var responseEntity = mock(ResponseEntity.class);
				var body = new AntragraumproxyGrpcSendRueckfrageAnswerResponse();

				when(bodyUriSpec.uri(anyString())).thenReturn(bodyContentTypeSpec);
				when(bodyContentTypeSpec.contentType(any(MediaType.class))).thenReturn(bodySpec);
				when(bodySpec.body(any(Object.class))).thenReturn(retrieveSpec);
				when(retrieveSpec.retrieve()).thenReturn(responseSpec);
				when(responseSpec.toEntity(any(Class.class))).thenReturn(responseEntity);
				when(responseEntity.getBody()).thenReturn(body);
				when(nachrichtRestClient.post()).thenReturn(bodyUriSpec);
			}

			@Test
			void shouldGetAnswerResponse() {
				restClient.answerRueckfrage(AntragraumproxyGrpcRueckfrageTestFactory.createRueckfrageAnswer(), TOKEN, ADDRESS);

				verify(nachrichtRestClient).post();
			}
		}

		@Nested
		class TestFindRueckfragen {
			@SuppressWarnings("unchecked")
			@BeforeEach
			void setup() {
				var uri = mock(RestClient.RequestHeadersUriSpec.class);
				var uriHeaders = mock(RestClient.RequestHeadersUriSpec.class);
				var response = mock(RestClient.ResponseSpec.class);
				var responseEntity = mock(ResponseEntity.class);
				var body = new AntragraumproxyGrpcFindRueckfragenResponse();

				when(uri.uri(anyString(), any(Object[].class))).thenReturn(uriHeaders);
				when(uriHeaders.retrieve()).thenReturn(response);
				when(response.toEntity(any(Class.class))).thenReturn(responseEntity);
				when(responseEntity.getBody()).thenReturn(body);
				when(nachrichtRestClient.get()).thenReturn(uri);
			}

			@Test
			void shouldCallClientFindRueckfragen() {
				restClient.getFindRueckfragen(ReplyNachrichtTestFactory.ID, TOKEN, ADDRESS);

				verify(nachrichtRestClient).get();
			}
		}

		@Nested
		class TestGetRueckfrage {
			@SuppressWarnings("unchecked")
			@BeforeEach
			void setup() {
				var uri = mock(RestClient.RequestHeadersUriSpec.class);
				var uriHeaders = mock(RestClient.RequestHeadersUriSpec.class);
				var response = mock(RestClient.ResponseSpec.class);
				var responseEntity = mock(ResponseEntity.class);
				var body = new AntragraumproxyGrpcGetRueckfrageResponse();

				when(uri.uri(anyString(), any(Object[].class))).thenReturn(uriHeaders);
				when(uriHeaders.retrieve()).thenReturn(response);
				when(response.toEntity(any(Class.class))).thenReturn(responseEntity);
				when(responseEntity.getBody()).thenReturn(body);
				when(nachrichtRestClient.get()).thenReturn(uri);
			}

			@Test
			void shouldCallClientGetRueckfrage() {
				restClient.getGetRueckfrage(ReplyNachrichtTestFactory.ID, TOKEN, ADDRESS);

				verify(nachrichtRestClient).get();
			}
		}
	}

}