/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static org.mockito.Mockito.*;

import org.mapstruct.factory.Mappers;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.client.RestClient;

import de.ozgcloud.antragsraum.attachments.FileService;
import de.ozgcloud.antragsraum.callcontext.ContextService;
import de.ozgcloud.antragsraum.common.AuthenticationHelper;

@Configuration
public class NachrichtenServiceGrpcTestConfiguration {
	@Bean
	NachrichtenRemoteService nachrichtRemoteService() {
		return new NachrichtenRemoteService(nachrichtenRestClient(), contextService(), nachrichtMapper(), antragraumProxyMapper());
	}

	NachrichtenRestClient nachrichtenRestClient() {
		return new NachrichtenRestClient(RestClient.builder().baseUrl("http://localhost:8080").build());
	}

	NachrichtMapper nachrichtMapper() {
		return new NachrichtMapper(mock(FileService.class));
	}

	AntragraumProxyMapper antragraumProxyMapper() {
		return Mappers.getMapper(AntragraumProxyMapper.class);
	}

	@Bean
	public ContextService contextService() {
		return new ContextService(authenticationHelper());
	}

	@Bean
	public AuthenticationHelper authenticationHelper() {
		return new AuthenticationHelper();
	}
}
